//Purpose:
//Updates the Assigned Resources Join table of a Service Appointment
//based on the selected resources in DayBack
 
//Action Type: Custom Event Action - On Event Save
//Prevent Default Action: Yes
 
//More info on custom event actions here:
//https://docs.dayback.com/article/20-event-actions
 
 
var changedResources = {};
 
//if resources have changed, get resource ids from resources loaded at start-up
if ( changesObject.resource ) {
  var resources = seedcodeCalendar.get('resources');
  //build object of changes resources by id
  for ( var i = 0 ; i < changesObject.resource.length ; i++){
    for ( var r = 0 ; r < resources.length ; r++){
      if(changesObject.resource[i]===resources[r].name){
        changedResources[resources[r].id] = resources[r].name;
      }
    }
  }
 
  //editing or creating an appointment
  if(event.eventID){
    //editing, we need to start by checking what the current resources are
    getCurrentAssigned(event.eventID);
  }
  else{
    //create the appointment now as it needs to exist for the join record to be created
    //surpress message toast so it only runs after this entire action is completed.
    config.suppressEditEventMessages = true;
    action.preventAction = true;
    dbk.updateEvent(event,changesObject,null,processUpdate);
  }
 
}
else {
  action.callbacks.confirm();
}
 
//process creation of new appointment
function processUpdate(data){
    //make sure creation was succesful
    if(data && data.eventID){
      //remove parentId from changesObject as this was set on creation and can't be edited.
      if(changesObject.contactID){
        delete changesObject.contactID;
      }
      //now that we have an appointment, we can create our join record(s)
      createAssignedResources(data.eventID, changedResources);
    }
    else{
      //let change go through as is for stock error capture
      config.suppressEditEventMessages = false;
      action.callbacks.confirm();
    }
}
 
//create join records for each resource
function createAssignedResources(appointmentId, resources) {
  var numberOfResources = Object.keys(resources).length;
  var responses = 0;
  for ( var resourceId in resources ){
    createAssignedResource(appointmentId, resourceId, processNewAssignment);
  }
 
  //callback for POSTING Join Records
  function processNewAssignment(data){
    //count our responses
    responses++;
    if(data && data.status===201){
      //all posts succesful
      if(responses===numberOfResources){
        //all posts succesful, re-enable toast notifications and complete action
        config.suppressEditEventMessages = false;
        action.preventAction = false;
        action.callbacks.confirm();
      }
    }
    else{
      alert("There was an error updating the resources. Please try again. If the problem persists, please contact support.");
      action.callbacks.cancel();
    }
  }
}
 
function getCurrentAssigned(appointmentId) {
  //check existing resources assigned
 
  //retrieve our canvas client object for authentication
  var client = fbk.client();
 
  //retrieve our canvas context object for REST links/urls
  var context = fbk.context();
 
  //retrieve the query URL from context
  var url = context.links.queryUrl;
 
  //SOQL Select Statement Fields;
  var select = 'SELECT+ServiceResourceId,Id+FROM+AssignedResource';
 
  //SOQL Where Clause
  var where = 'WHERE+ServiceAppointmentId=\'' + appointmentId + '\'';
 
  //SOQL query
  var query = select + '+' + where;
 
  //final URL for GET
  var finalUrl = url + '?q=' + query;
 
  //build settings object for Ajax call to Salesforce
  var settings = {};
  settings.client = client;
  settings.contentType = 'application/json';
  settings.success = processResult;
 
  //Use canvas function to query
  Sfdc.canvas.client.ajax(finalUrl, settings);
 
  function processResult(data) {
    if (data.status === 200) {
      if (data.payload && data.payload.totalSize === 0) {
        //no existing assigned resourcew, create them.
        createAssignedResources(appointmentId, changedResources);
      }
      else {
        //update existing join records
        updateAssignedResources(data.payload.records,changedResources);
      }
    }
    else{
      alert('There was an error updating the resources. Please try again. If the problem persists, please contact support.');
      action.callbacks.cancel();
    }
  }
}
 
 
 
//update the join records
function updateAssignedResources(current,resources){
  //remove any requests that already exist
  var c;
  var cleanedCurrent = [];
  for (c = 0 ; c < current.length ; c++){
    if(resources[current[c].ServiceResourceId]){
      delete resources[current[c].ServiceResourceId];
    }
    else{
      cleanedCurrent.push(current[c]);
    }
  }
  current = cleanedCurrent;
  var numberOfResources = Object.keys(resources).length;
  var responses = 0;
  var keys = Object.keys(resources);
  var r;
 
  if(current.length===keys.length){
    //number of assigned records is unchanged, just need to update the existing records
    for ( c = 0 ; c < current.length ; c++){
      updateAssignedResource(current[c].Id,keys[c],processRequests);
    }
  }
  else if(current.length < keys.length){
    //we have new resources to add
    for ( r = 0 ; r < keys.length ; r++){
      if(current.length < r){
        updateAssignedResource(current[r].Id,keys[r],processRequests);
      }
      else{
        createAssignedResource(event.eventID,keys[r],processRequests);
      }
    }
  }
  else if(current.length > keys.length){
    //we have removed resources, so need to delete some existing assignments
    for ( c = 0 ; c < current.length ; c++){
      if(c < resources.length){
        updateAssignedResource(current[c].Id,keys[c],processRequests);
      }
      else{
        deleteAssignedResource(current[c].Id,processRequests);
      }
    }
  }
 
  function processRequests(data){
    responses++;
    if(data && ( data.status===204 || data.status===201  )){
      //all requests succesful
      if(responses>=numberOfResources){
        //all posts succesful, re-enable toast notifications and complete action
        config.suppressEditEventMessages = false;
        action.preventAction = false;
        action.callbacks.confirm();
      }
    }
    else{
      alert('There was an error updating the resources. Please try again. If the problem persists, please contact support.');
      action.callbacks.cancel();
    }
  }
}
 
//function for creating individual join records
function createAssignedResource(appointmentId, resourceId, callback) {
  var objectName = 'AssignedResource';
  var request = {};
  request.ServiceAppointmentId = appointmentId;
  request.ServiceResourceId = resourceId;
  //retrieve our canvas client object for authentication
  var client = fbk.client();
  //retrieve our canvas context object for REST links/urls
  var context = fbk.context();
  var url = context.links.sobjectUrl + objectName + '/';
  //settings object for our POST
  var settings = {};
  settings.client = client;
  settings.method = 'POST';
  settings.contentType = 'application/json';
  settings.data = JSON.stringify(request);
  settings.success = callback;
  //make canvas AJAX call to salesforce for this edit
  Sfdc.canvas.client.ajax(url, settings);
}
 
function updateAssignedResource(Id,resourceId,callback) {
  var objectName = 'AssignedResource';
  var request = {};
  request.ServiceResourceId = resourceId;
  //retrieve our canvas client object for authentication
  var client = fbk.client();
  //retrieve our canvas context object for REST links/urls
  var context = fbk.context();
  var url = context.links.sobjectUrl + objectName + '/' + Id;
  //settings object for our POST
  var settings = {};
  settings.client = client;
  settings.method = 'PATCH';
  settings.contentType = 'application/json';
  settings.data = JSON.stringify(request);
  settings.success = callback;
  //make canvas AJAX call to salesforce for this edit
  Sfdc.canvas.client.ajax(url, settings);
}
 
function deleteAssignedResource(Id,callback){
  var objectName = 'AssignedResource';
  //retrieve our canvas client object for authentication
  var client = fbk.client();
  //retrieve our canvas context object for REST links/urls
  var context = fbk.context();
  var url = context.links.sobjectUrl + objectName + '/' + Id;
  //settings object for our POST
  var settings = {};
  settings.client = client;
  settings.method = 'DELETE';
  settings.contentType = 'application/json';
  settings.success = callback;
  //make canvas AJAX call to salesforce for this edit
  Sfdc.canvas.client.ajax(url, settings);
}